//************************************************************************//
//									  //
//  Copyright 2013 Bertram Kopf (bertram@ep1.rub.de)			  //
//  	      	   Julian Pychy (julian@ep1.rub.de)			  //
//          	   - Ruhr-Universität Bochum 				  //
//									  //
//  This file is part of Pawian.					  //
//									  //
//  Pawian is free software: you can redistribute it and/or modify	  //
//  it under the terms of the GNU General Public License as published by  //
//  the Free Software Foundation, either version 3 of the License, or 	  //
//  (at your option) any later version.	 	      	  	   	  //
//									  //
//  Pawian is distributed in the hope that it will be useful,		  //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of	  //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	  //
//  GNU General Public License for more details.	      		  //
//									  //
//  You should have received a copy of the GNU General Public License     //
//  along with Pawian.  If not, see <http://www.gnu.org/licenses/>.	  //
//									  //
//************************************************************************//

// BlattWBarrierDynamics class definition file. -*- C++ -*-
// Copyright 2013 Bertram Kopf

#pragma once

#include <iostream>
#include <vector>
#include <complex>
#include <map>
#include <string>
#include <memory>

#include "PwaUtils/AbsDynamics.hh"
#include "PwaDynamics/RadMultipoleFormFactor.hh"
#include "PwaDynamics/BarrierFactor.hh"

class AbsPawianParameters;

class RadM1Dynamics : public AbsDynamics{

public:
  RadM1Dynamics(std::string& name, std::vector<Particle*>& fsParticles, Particle* mother, std::vector<Particle*>& fsParticlesDaughter1, std::vector<Particle*>& fsParticlesDaughter2, const std::string& wignerDKey, double qR, double massB=1., bool useKedr=false);
  virtual ~RadM1Dynamics();

  virtual std::string type() {return "RadM1Dynamics";}
  virtual complex<double> eval(EvtData* theData, AbsXdecAmp* grandmaAmp, Spin OrbMom=0);

  virtual void fillDefaultParams(std::shared_ptr<AbsPawianParameters> fitPar);
  virtual void fillParamNameList();

  virtual void updateFitParams(std::shared_ptr<AbsPawianParameters> fitPar);
  virtual void setMassKey(std::string& theMassKey);
  
  virtual void fillMasses(EvtData* theData);  

protected:
  std::vector<Particle*> _fsParticlesDaughter1;
  std::vector<Particle*> _fsParticlesDaughter2;
  std::string _dynMassKeyDaughter1;
  std::string _dynMassKeyDaughter2;
  std::string _dynEgammaCMmotherKey;
  unsigned short _dynMassIdDaughter1;
  unsigned short _dynMassIdDaughter2;
  unsigned short _dynEgammaCMmotherId;
  bool _isP1Gamma;
  double _massB0;
  
  std::string _wignerDKey;
  std::string _wignerDqNormKey;
  unsigned short _wignerDqId;
  unsigned short _wignerDqNormId;
  double _qR;
  bool _fitqRVals;
  bool _useKedr;
  std::string _fitqRKey;

private:

};
